-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Nest:StartActivity()
   	print("START! -- Nest:StartActivity()!");
	
	self.SweepZone = SceneMan.Scene:GetArea("SweepZone"); -- Zone to clear out of enemies

	self.WindToRight = SceneMan.Scene:GetArea("WindToRight");
	self.WindToLeft = SceneMan.Scene:GetArea("WindToLeft");
	self.RocketLZ  = SceneMan.Scene:GetArea("RocketLZ");
	
	self.braindead = {};
	self.AssignedBrains = {};
	self.MOIDLimit = 200;
    
    self.GenericTimer = Timer();

   	self.CoalitionSpawnTimer = Timer();
	self.CoalitionSpawnInterval = 10000;
	
	self.SakuraKillTimer = Timer();
	self.SakuraToKillInterval = 1500;
	self.Sakura30Thrown = false;
	
    self.WeaponSets = { Light = 0 , Heavy = 1 , Extreme = 2};
    self.FightPhase = { NOFIGHT = 0 , WAITSHIP = 1 , SHIPLANDING = 2 , SHIPBROKEN = 3 , EVACUATEDROPSHIP = 4 , EVACUATEROCKET = 5};
    self.CurrentFightPhase = self.FightPhase.NOFIGHT;

    self.CoalitionLightWeapons = {"Shotgun" , "Compact Assault Rifle" , "Flamer"};
    self.CoalitionHeavyWeapons = {"Auto Shotgun" , "Assault Rifle" , "Napalm Flamer"};
    self.CoalitionExtremeWeapons = {"Auto Shotgun" , "Assault Rifle" , "Uber Cannon" , "Rocket Launcher"};
    self.CoalitionSecondaryWeapons = {"Auto Pistol"};
	
	-- Generic timing and money parameters
	-- No money and alive reinforcements availabe for player according to the plot
	self.PlayerGold = 0; -- Player starting credits
	
	-- Ronins
	-- Dafred - Awesome - Commander
	-- Mia - Stealthy, runs fast - Specops
	-- Dimitry - Tall, see further - Sniper
	-- Brutus - Heavy and slow - Heavy weapons
	-- Sandra - Sexy - Medic
	-- Gordon - Armor vs Impact - Grenadier
	
	-- Ronin squad
	self.Mia = nil;
	self.Dimitri = nil;

	-- Repair robot
	self.Robot = nil;
	
	self.ChunksToRepair = 8;
	self.ChunksRepaired = 0;
	self.ChunkRepairTime = 10000;
	self.RepairTimer = Timer();
	self.BeamTarget = 0;
	self.EnableBeams = false;
	
	-- Beam bombardement
	self.BeamTimer = Timer();
	self.BeamWarningInterval = 9000;
	self.BeamFireInterval = 6000;
	self.BeamTargeted = false;
	
	self.BeamEmitter = CreateAEmitter("Particle Cannon Beam" , "Nest.rte");
	self.BeamEmitter.RotAngle = math.rad(270);
	--self.BeamEmitter.Pos = Vector (1200 , 1200);
	self.BeamEmitter:EnableEmission(false);
	MovableMan:AddParticle(self.BeamEmitter);
	
	self.BeamEmitterTimer = Timer();
	self.BeamEmitterTime = 500;

	-- Rocket for those who lost dropship
	self.Rocket = nil;
	self.RocketTimer = Timer();
	self.RocketInterval = 10000;
	
	self.Orbited = false;
	
	-- Dropship and engine emissions
	self.DropShip = nil;
	
	-- Timer to trigger health drain if robot was killed
	self.DropShipHealthDrainTimer = Timer();

	-- Create smoke emitters
	self.LeftEmitter = CreateAEmitter("Engine Smoke" , "Nest.rte");
	-- self.LeftEmitter.Pos = Vector(150 , 2000);
	self.LeftEmitter:EnableEmission(false);
	-- MovableMan:AddParticle(self.LeftEmitter);
	
	self.RightEmitter = CreateAEmitter("Engine Smoke" , "Nest.rte");
	-- self.RightEmitter.Pos = Vector(350 , 2000);
	self.RightEmitter:EnableEmission(false);
	-- MovableMan:AddParticle(self.RightEmitter);
	
	-- Flag to check if ship is low enough to show false evac objectives
	self.LandingTriggered = false;
	
	-- Flag if we sent a repair bot
	self.RepairBotSent = false;

    -- Wind
    self.Wind = {};
    
    for i = 0 , 20 do
		self.Wind[i] = CreateAEmitter("Wind Emitter" , "Nest.rte");
		self.Wind[i].Pos = self.WindToRight:GetRandomPoint();
		self.Wind[i]:EnableEmission(true);
		MovableMan:AddParticle(self.Wind[i]);
    end
    
    self.WindTimer = Timer();
	
	-- Ronin special abilities countdonwn timer
	self.SpecialAbilitiesTimer = Timer();
	self.SpecialAbilitiesTimer:Reset();
	self.SpecialAbilitiesInterval = 50; -- Time to refresh special abilities
	
   	-- Set up teams
   	-- Team 2 is always CPU
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

    -- Set up AI modes for the Actors that have been added to the scene by the scene definition
    for actor in MovableMan.AddedActors do
		actor.AIMode = Actor.AIMODE_SENTRY;
    end

   	-- Set up players
    for player = 0, self.PlayerCount - 1 do
        -- Check if we already have a brain assigned
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			local brain = CreateAHuman("Dafred Brain" , "Nest.rte");
			brain.Pos = Vector(400 + player * 30, 2100);
			brain:AddInventoryItem(CreateHDFirearm("Glock","Ronin.rte"));
			brain.AIMode = Actor.AIMODE_GOTO;
			brain:ClearAIWaypoints();
			brain:AddAISceneWaypoint(Vector(2000  + player * 30 , 2100));

			brain.Team = self.PlayerTeam;
			self.AssignedBrains[player] = brain;
            -- Let the spawn into the world, passing ownership
            MovableMan:AddActor(brain);
            -- Set the found brain to be the selected actor at start
            self:SetPlayerBrain(brain, player);
			self:SwitchToActor(brain, player, self:GetTeamOfPlayer(player));
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            self:SetLandingZone(self:GetPlayerBrain(player).Pos, player);
            -- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
        end
    end
    
    -- Spawn ronin squad
	self.Dimitri = CreateAHuman("Dimitri" , "Nest.rte");
	self.Dimitri.Pos = Vector(2550 , 35);
	self.Dimitri:AddInventoryItem(CreateHDFirearm("Sakura x30","Nest.rte"));
	self.Dimitri:AddInventoryItem(CreateHDFirearm("Glock","Ronin.rte"));
	self.Dimitri.Team = self.PlayerTeam;
    MovableMan:AddActor(self.Dimitri);

	self:SwitchToActor(self.Dimitri, 0 , self:GetTeamOfPlayer(0));

	self.Mia = CreateAHuman("Mia" , "Nest.rte");
	self.Mia.Pos = Vector(50 , 2100);
	self.Mia:AddInventoryItem(CreateHDFirearm("Glock","Ronin.rte"));
	self.Mia:AddInventoryItem(CreateHDFirearm("Medikit","Nest.rte"));
	self.Mia.AIMode = Actor.AIMODE_GOTO;
	self.Mia:ClearAIWaypoints();
	self.Mia:AddAISceneWaypoint(Vector(1800 , 2100));
	self.Mia.Team = self.PlayerTeam;
    MovableMan:AddActor(self.Mia);
	
   
	--[[self.Mia.Health = 30; -- Medikit debug with low healths
	self.Dimitri.Health = 30;
	self.Brutus.Health = 30;
	self.Sandra.Health = 30;
	self.Gordon.Health = 30;]]--


    -- Set the team's funds.
    self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);

    -- Reset all timers
	self.GenericTimer:Reset();
	
	-- self:FireBeam(1200); -- debug
end

-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function Nest:PauseActivity(pause)
    print("PAUSE! -- DummyAssault:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function Nest:EndActivity()
    print("END! -- DummyAssault:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Nest:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					FrameMan:SetScreenText("Commander died!", player, 333, -1, false);
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					--if not MovableMan:GetClosestBrainActor(team) then
						self.WinnerTeam = self:OtherTeam(team);
						ActivityMan:EndActivity();
					--end
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Skip phases
-----------------------------------------------------------------------------------------
function Nest:SkipPhases()
	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		self:AdvanceToWaitShip();
	end
	
	if self.CurrentFightPhase == self.FightPhase.WAITSHIP then
		self:AdvanceToShipLanding();
	end
	
	--if self.CurrentFightPhase == self.FightPhase.SHIPLANDING then
	--	self:AdvanceToShipBroken();
	--end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Nest:UpdateActivity()
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();

	-- self:SkipPhases();  -- Debug only!
	self:DoBrainSelection();
	self:DoSpecialAbilities();
	self:ReassignActors();
	self:DoWind();
	
	--self:SkipPhases();
	
	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		self:PhaseNoFight();
	elseif self.CurrentFightPhase == self.FightPhase.WAITSHIP then
		self:PhaseWaitShip();
	elseif self.CurrentFightPhase == self.FightPhase.SHIPLANDING then
		self:PhaseShipLanding();
	elseif self.CurrentFightPhase == self.FightPhase.SHIPBROKEN then
		self:PhaseShipBroken();
	elseif self.CurrentFightPhase == self.FightPhase.EVACUATEDROPSHIP then
		self:PhaseEvacuateDropShip();
	elseif self.CurrentFightPhase == self.FightPhase.EVACUATEROCKET then
		self:PhaseEvacuateRocket();
	end
	
	if self.Mia ~= nil and MovableMan:IsActor(self.Mia) then
		if self.Mia.Health < 1 then
			self.Mia = nil;
		end
	end

	if self.Dimitri ~= nil and MovableMan:IsActor(self.Dimitri) then
		if self.Dimitri.Health < 1 then
			self.Dimitri = nil;
		end
	end
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Do Wind
-----------------------------------------------------------------------------------------
function Nest:DoWind()
	-- Move actors due to wind force
	for actor in MovableMan.Actors do
		if self.WindToRight:IsInside(actor.Pos) then
			actor.Vel = actor.Vel + Vector(0.07 , 0);
		end

		if actor.PresetName ~= "Rocket MK1" and actor.PresetName ~= "Drop Ship MK1" then
			if self.WindToLeft:IsInside(actor.Pos) then
				actor.Vel = actor.Vel + Vector(-0.15 , 0);
			end
		else
			if self.WindToLeft:IsInside(actor.Pos) then
				actor.Vel = actor.Vel + Vector(0.07 , 0);
			end
		end
		
		if actor.PresetName == "Rocket MK1" then
			actor.Vel = actor.Vel + Vector(0 , 0.08);
			if actor.Vel.Y < -6 then
				actor.Vel.Y = -6;
			end
		elseif actor.PresetName == "Drop Ship MK1" then
			actor.Vel = actor.Vel + Vector(0 , 0.05);
			if actor.Vel.Y < -6 then
				actor.Vel.Y = -6;
			end
		end
	end
	
	-- Change wind particles emitters
	if self.WindTimer:IsPastSimMS(50) then
		for i = 0 , 20 do
			-- self.Wind[math.random(#self.Wind)].Pos = self.WindZone:GetRandomPoint();
			self.Wind[i].Pos = self.WindToRight:GetRandomPoint();
		end
		self.WindTimer:Reset();
	end
end
-----------------------------------------------------------------------------------------
-- Special abilities
-----------------------------------------------------------------------------------------
function Nest:DoSpecialAbilities()
	-- Check if some smart guy threw sakura to the friends below
	if MovableMan:IsActor(self.Dimitri) then
		if self.Sakura30Thrown then
			if self.SakuraKillTimer:IsPastSimMS(self.SakuraToKillInterval) then
				for item in MovableMan.Items do
					if item.PresetName == "Sakura x30" and not self.Dimitri:HasObject("Sakura x30") then
						item.ToSettle = true;
					end
				end --for
			end --if
		else
			if self.Dimitri:HasObject("Sakura x30") then
				self.Sakura30Thrown = false;
			else
				self.Sakura30Thrown = true;
				if not self.Sakura15Thrown then
					self.SakuraKillTimer:Reset();
				end
			end
		end --else
	end
end
-----------------------------------------------------------------------------------------
-- Spawn coalition trooper
-----------------------------------------------------------------------------------------
function Nest:SpawnClone(weaponset)
	if MovableMan:GetMOIDCount() <= self.MOIDLimit then
		print ("Spawn clone");
		local actor = nil;
		
		--if weaponset == self.WeaponSets.Light then
			actor = CreateAHuman("Soldier Light" , "Coalition.rte");
		--else
		--	actor = CreateAHuman("Soldier Heavy" , "Coalition.rte");
		--end
		
		if weaponset == self.WeaponSets.Light then
			actor:AddInventoryItem(CreateHDFirearm(self.CoalitionLightWeapons[math.random(#self.CoalitionLightWeapons)],"Coalition.rte"));
		elseif weaponset == self.WeaponSets.Heavy then
			actor:AddInventoryItem(CreateHDFirearm(self.CoalitionHeavyWeapons[math.random(#self.CoalitionHeavyWeapons)],"Coalition.rte"));
		elseif weaponset == self.WeaponSets.Extreme then
			actor:AddInventoryItem(CreateHDFirearm(self.CoalitionExtremeWeapons[math.random(#self.CoalitionExtremeWeapons)],"Coalition.rte"));
		end
		-- actor:AddInventoryItem(CreateHDFirearm(self.CoalitionSecondaryWeapons[math.random(#self.CoalitionSecondaryWeapons)],"Coalition.rte"));

		actor.Team = self.CPUTeam;
		actor.AIMode = Actor.AIMODE_BRAINHUNT;
		actor.Pos = Vector(10 , 2100);
		MovableMan:AddActor(actor);
	end
end
-----------------------------------------------------------------------------------------
-- Spawn dropship
-----------------------------------------------------------------------------------------
function Nest:SpawnDropship()
	-- Add escape dropship
	print("Spawn drop ship");				
	
	self.DropShip = CreateACDropShip("Drop Ship MK1" , "Base.rte");
	self.DropShip.Pos = Vector(2000 , -20);
	--self.DropShip.Pos = Vector(2000 , 1500); --Debug
	self.DropShip.Team = self.PlayerTeam;
	
	self.DropShip:SetControllerMode(Controller.CIM_AI, -1);
	
	MovableMan:AddActor(self.DropShip);
	print("Drop ship spawn complete");
end
-----------------------------------------------------------------------------------------
-- Spawn rocket
-----------------------------------------------------------------------------------------
function Nest:SpawnRocket()
	-- Add escape rocket
	print("Spawn rocket");				
	
	self.Rocket = CreateACRocket("Rocket MK1" , "Base.rte");
	self.Rocket.Pos = Vector(self.RocketLZ:GetRandomPoint().X , -10);
	self.Rocket.Team = self.PlayerTeam;
	
	self.Rocket:SetControllerMode(Controller.CIM_AI, -1);
	
	MovableMan:AddActor(self.Rocket);
	print("Rocket spawn complete");
end
-----------------------------------------------------------------------------------------
-- Spawn repair bot
-----------------------------------------------------------------------------------------
function Nest:SpawnRobot()
	print("Spawn robot crate");				
	local ship = CreateACRocket("Drop Crate","Dummy.rte");
	ship.Team = self.PlayerTeam;

	self.Robot = CreateAHuman("Robot 1")
	self.Robot.Team = self.PlayerTeam;
	self.Robot.AIMode = Actor.AIMODE_SENTRY;
	self.Robot:AddInventoryItem(CreateHDFirearm("Blaster Pistol","Base.rte"));
	ship:AddInventoryItem(self.Robot);
	
	ship.Pos = Vector(650 , -20);
	MovableMan:AddActor(ship);
end
-----------------------------------------------------------------------------------------
-- Rassign actors as the variables gets nillfied once troops boarded to ship
-----------------------------------------------------------------------------------------
function Nest:ReassignActors()
	-- Reassign actors as the variables gets nillfied once troops boarded to ship
	local miafound = false;
	local dimitrifound = false;
	local robotfound = false;
	local brainfound = {};
	local foundbrainscount = 0;

	for player = 0, self.PlayerCount - 1 do
		brainfound[player] = false;
	end
	
	for actor in MovableMan.Actors do
		if actor.PresetName == "Dafred Brain" then
			brainfound[foundbrainscount] = true;
			self.AssignedBrains[foundbrainscount] = actor;
			foundbrainscount = foundbrainscount + 1;
		end

		if actor.PresetName == "Mia" then
			self.Mia = actor;
			miafound = true;
		end

		if actor.PresetName == "Robot 1" then
			self.Robot = actor;
			robotfound = true;
		end
		
		if actor.PresetName == "Dimitri" then
			self.Dimitri = actor;
			dimitrifound = true;
		end
	end

	if not dimitrifound then
		self.Dimitri = nil;
	end

	if not miafound then
		self.Mia = nil;
	end

	if not robotfound then
		self.Robot = nil;
	end

	for player = 0, self.PlayerCount - 1 do
		if not brainfound[player] then
			self.AssignedBrains[player] = nil;
		end
	end
end
-----------------------------------------------------------------------------------------
-- Operate broken ship so it can't fly away
-----------------------------------------------------------------------------------------
function Nest:ControlShip()
	if MovableMan:IsActor(self.DropShip) then
		-- For some reason game B23 craches when ship exits the area due to this routine
		if self.DropShip.Pos.X > 1 then
			-- Calculate altitude
			local desiredalt = SceneMan:MovePointToGround(self.DropShip.Pos , 20 , 3).Y;
			local alt = SceneMan:FindAltitude(self.DropShip.Pos , 0 , 3);
			
			-- self.DropShip:SetControllerMode(Controller.CIM_NETWORK, -1);

			self.LeftEmitter.Pos = self.DropShip.Pos + Vector(- 70 , 0);
			self.RightEmitter.Pos = self.DropShip.Pos + Vector( 70 , 0);
		
			if alt < 100 then
				self.DropShip:GetController():SetState(Controller.MOVE_DOWN , false);
				self.DropShip:GetController():SetState(Controller.MOVE_UP , true);
			elseif alt > 400 then
				self.DropShip.Vel = self.DropShip.Vel + Vector(0 , 0.05);
				self.DropShip:GetController():SetState(Controller.MOVE_DOWN , true);
				self.DropShip:GetController():SetState(Controller.MOVE_UP , false);
			end
		else	
			self.LeftEmitter:EnableEmission(false);
			self.RightEmitter:EnableEmission(false);		
		end
	end
end
-----------------------------------------------------------------------------------------
-- Fire lazer beam to the specified Vector target
-----------------------------------------------------------------------------------------
function Nest:FireBeam(target)
	local beamblast = CreateTDExplosive("Base Particle" , "Nest.rte");
	beamblast.Vel = Vector(0 , 400);
	beamblast.Pos = Vector(target , 0);
	MovableMan:AddItem(beamblast);
	
	self.BeamEmitter.Pos = Vector(target , -40);
	self.BeamEmitter:TriggerBurst();
	self.BeamEmitter:EnableEmission(true);
	self.BeamEmitterTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- Phase "No fight"
-----------------------------------------------------------------------------------------
function Nest:PhaseNoFight()
	-- Show team marks and special abilities
	if not self.GenericTimer:IsPastSimMS(20000) then
		if self.Mia ~= nil and MovableMan:IsActor(self.Mia) then
			self:AddObjectivePoint("Mia - Stealth", self.Mia.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Dimitri ~= nil and MovableMan:IsActor(self.Dimitri) then
			self:AddObjectivePoint("Dimitry - Sniper", self.Dimitri.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("Dafred - Commander - Best", self.AssignedBrains[player].Pos + Vector(0 , -35), self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	elseif not self.GenericTimer:IsPastSimMS(40000) then
		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("PROTECT", self.AssignedBrains[player].AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	end
	
	-- Spawn enemies
	if self.GenericTimer:IsPastSimMS(6000) then
		if self.CoalitionSpawnTimer:IsPastSimMS(self.CoalitionSpawnInterval) then
			self:SpawnClone(self.WeaponSets.Light);
			self.CoalitionSpawnTimer:Reset();
		end
	end

	-- Next phase after some shooting
	if self.GenericTimer:IsPastSimMS(140000) then
		self:AdvanceToWaitShip();
	end
end -- func
-----------------------------------------------------------------------------------------
-- "No fight" -> "Wait ship"
-----------------------------------------------------------------------------------------
function Nest:AdvanceToWaitShip()
	print("Secure LZ");

	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText("SECURE LZ AND WAIT FOR DROPSHIP", player, 0, 10000, true);
	end

	self.CoalitionSpawnInterval = 10000;

	self.GenericTimer:Reset();
	self.CurrentFightPhase = self.FightPhase.WAITSHIP;
end
-----------------------------------------------------------------------------------------
-- Phase "Wait ship"
-----------------------------------------------------------------------------------------
function Nest:PhaseWaitShip()
	-- Show team marks and special abilities
	if self.GenericTimer:IsPastSimMS(5000) and not self.GenericTimer:IsPastSimMS(16000) then
		if self.CoalitionSpawnTimer:IsPastSimMS(self.CoalitionSpawnInterval) then
			self:SpawnClone(self.WeaponSets.Light);
			self.CoalitionSpawnTimer:Reset();
		end
	end

	-- Spawn a horde of enemies and then reduce their amount
	if not self.GenericTimer:IsPastSimMS(16000) then
		self.CoalitionSpawnInterval = 2000;
	else
		self.CoalitionSpawnInterval = 16000;
	end

	local liveenemies = 0;
	for actor in MovableMan.Actors do
		if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam and self.SweepZone:IsInside(actor.Pos) then
			liveenemies = liveenemies + 1;
			
			if not self.GenericTimer:IsPastSimMS(10000) then
				self:AddObjectivePoint("Kill", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	end -- for
	
	if liveenemies == 0 then
		self:AdvanceToShipLanding();
	end -- if
end -- func
-----------------------------------------------------------------------------------------
-- "Wait ship" -> "Ship landing"
-----------------------------------------------------------------------------------------
function Nest:AdvanceToShipLanding()
	print("Ship landing");

	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText("DROPSHIP ARRIVING, DON'T WASTE IT!!!", player, 0, 10000, true);
	end

	self.CoalitionSpawnInterval = 9500;		

	self.GenericTimer:Reset();
	self.CurrentFightPhase = self.FightPhase.SHIPLANDING;
	
	self:SpawnDropship();
end
-----------------------------------------------------------------------------------------
-- Phase "Ship landing"
-----------------------------------------------------------------------------------------
function Nest:PhaseShipLanding()
	-- Spawn enemies
	if self.GenericTimer:IsPastSimMS(5000) then
		if self.CoalitionSpawnTimer:IsPastSimMS(self.CoalitionSpawnInterval) then
			self:SpawnClone(self.WeaponSets.Light);
			self.CoalitionSpawnTimer:Reset();
		end
	end
	
	-- Show dropship protection marks
	if MovableMan:IsActor(self.DropShip) then
		if not self.GenericTimer:IsPastSimMS(10000) then
			self:AddObjectivePoint("PROTECT", self.DropShip.AboveHUDPos , self.PlayerTeam, GameActivity.ARROWDOWN);
		end	
	end
	
	-- Check if dropship is low enough to show false evac objectives
	if not self.LandingTriggered then
		if MovableMan:IsActor(self.DropShip) then
			local alt = SceneMan:FindAltitude(self.DropShip.Pos , 0 , 3);
			
			if alt < 250 then
				self.LandingTriggered = true;
			end
		end
	end

	if not MovableMan:IsActor(self.DropShip) then
		self:AdvanceToEvacuateRocket("DROPSHIP LOST, WAIT FOR ROCKET");
	end
	
	-- Show false evac objectives
	if self.LandingTriggered then
		--if MovableMan:IsActor(self.Dimitri) then
		--	self:AddObjectivePoint("Evacuate", self.Dimitri.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		--end

		--if MovableMan:IsActor(self.Mia) then
		--	self:AddObjectivePoint("Evacuate", self.Mia.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		--end

		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("Evacuate", self.AssignedBrains[player].AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
		
		self:AddObjectivePoint("ESCAPE", Vector(650, 1), Activity.TEAM_1, GameActivity.ARROWUP);
		self:AddObjectivePoint("ESCAPE", Vector(1950, 1), Activity.TEAM_1, GameActivity.ARROWUP);		
		
		if MovableMan:IsActor(self.DropShip) then
			self:AddObjectivePoint("Board ship and evacuate", self.DropShip.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if MovableMan:IsActor(self.DropShip) then
			local alt = SceneMan:FindAltitude(self.DropShip.Pos , 0 , 3);
			
			if alt > 1200 then
				self:AdvanceToShipBroken();
			end
		end
	end
end -- func
-----------------------------------------------------------------------------------------
-- "Ship landing" -> "Ship broken"
-----------------------------------------------------------------------------------------
function Nest:AdvanceToShipBroken()
	print("Ship broken");

	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText("ENGINE FAILURE!!!", player, 250, 10000, true);
	end

	self.CoalitionSpawnInterval = 10000;		

	self.GenericTimer:Reset();
	self.CurrentFightPhase = self.FightPhase.SHIPBROKEN;
	
	-- Start smokes
	self.LeftEmitter.Pos = self.DropShip.Pos + Vector(- 70 , 0);
	self.LeftEmitter:EnableEmission(true);
	MovableMan:AddParticle(self.LeftEmitter);
	
	self.RightEmitter.Pos = self.DropShip.Pos + Vector( 70 , 0);
	self.RightEmitter:EnableEmission(true);
	MovableMan:AddParticle(self.RightEmitter);
	
	self.RepairTimer:Reset();
	
	if MovableMan:IsActor(self.Dropship) then
		self.DropShip.AIMode = Actor.AIMODE_STAY;
	end
end
-----------------------------------------------------------------------------------------
-- Phase "Ship broken"
-----------------------------------------------------------------------------------------
function Nest:PhaseShipBroken()
	-- Spawn enemies
	if self.CoalitionSpawnTimer:IsPastSimMS(self.CoalitionSpawnInterval) then
		self:SpawnClone(self.WeaponSets.Heavy);
		self.CoalitionSpawnTimer:Reset();
	end

	-- Spawn a horde of enemies and then reduce their amount
	if not self.GenericTimer:IsPastSimMS(14000) then
		self.CoalitionSpawnInterval = 2000;
	else
		self.CoalitionSpawnInterval = 10000;
	end

	-- Show HOLD banner
	if self.GenericTimer:IsPastSimMS(12000) and not self.GenericTimer:IsPastSimMS(14000) then
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("HOLD UNTIL WE SEND REPAIR ROBOT", player, 0, 8000, true);
		end
	end
	
	-- Do robot spawn if it is already not
	if not self.RepairBotSent then
		if self.GenericTimer:IsPastSimMS(26000) then
			self:SpawnRobot();
			for player = 0, self.PlayerCount - 1 do
				self:ResetMessageTimer(player);
				FrameMan:ClearScreenText(player);
				FrameMan:SetScreenText("ROBOT ARRIVING DON'T MISS IT", player, 500, 8000, true);
			end
			self.RepairBotSent = true;
		end
	else
		-- Show ship repair mark
		if MovableMan:IsActor(self.Robot) then
			self:AddObjectivePoint("Board to ship to start repair", self.Robot.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end
		
		-- Flag to 
		local robotalive = false;
		
		-- Show crate mark not to waste it
		-- and search all battle field for robot to chech if it still alive
		for actor in MovableMan.Actors do
			if actor.PresetName == "Drop Crate" then
				self:AddObjectivePoint("CATCH ROBOT", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
			
			if actor.PresetName == "Robot 1" then
				robotalive = true;
			end
			
			if actor:HasObject("Robot 1") then
				robotalive = true;
			end
		end
		
		if not robotalive then
			self:AdvanceToEvacuateRocket("ROBOT LOST , WAIT FOR ROCKET");
		end
	end
	
	self:ControlShip();
	
	-- Do dropship specific routines if it exists
	if MovableMan:IsActor(self.DropShip) then
		if not self.GenericTimer:IsPastSimMS(10000) then
			self:AddObjectivePoint("ENGINE FAILURE", self.DropShip.AboveHUDPos , self.PlayerTeam, GameActivity.ARROWDOWN);
		end	
		
		-- Process repairing
		if self.RepairTimer:IsPastSimMS(self.ChunkRepairTime) then
			if self.DropShip:HasObject("Robot 1") then
				if self.ChunksRepaired < self.ChunksToRepair then
					self.ChunksRepaired = self.ChunksRepaired + 1;
					
					for player = 0, self.PlayerCount - 1 do
						self:ResetMessageTimer(player);
						FrameMan:ClearScreenText(player);
						FrameMan:SetScreenText(string.format("REPAIRING... %d%% COMPLETE" , (self.ChunksRepaired - 1) / self.ChunksToRepair * 100) , player, 500, 3000, true);
					end					
				else
					self:AdvanceToEvacuateDropShip();
				end
			end
			self.RepairTimer:Reset();
		end
	else
		self:AdvanceToEvacuateRocket("DROPSHIP LOST, WAIT FOR ROCKET");
	end
end -- func
-----------------------------------------------------------------------------------------
-- "Ship broken" -> "Evacuate dropship" when dropship was fixed
-----------------------------------------------------------------------------------------
function Nest:AdvanceToEvacuateDropShip()
	-- Turn smoke off
	self.LeftEmitter:EnableEmission(false);
	self.RightEmitter:EnableEmission(false);

	self.CoalitionSpawnInterval = 6000;
	
	-- Immediately start ion bombing if brain is inside the ship
	if MovableMan:IsActor(self.DropShip) then
		if self.DropShip:HasObject("Dafred Brain") then
			self.BeamWarningInterval = 3000;
			self.BeamFireInterval = 6000;
		end
	end
	
	self.CurrentFightPhase = self.FightPhase.EVACUATEDROPSHIP;
	
	self.GenericTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- "Ship broken" -> "Evacuate Rocket" when dropship was wasted
-----------------------------------------------------------------------------------------
function Nest:AdvanceToEvacuateRocket(reason)
	-- self.LeftEmitter:EnableEmission(false);
	-- self.RightEmitter:EnableEmission(false);

	self.CoalitionSpawnInterval = 6000;	

	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText(reason, player, 500, 10000, true);
	end		

	self.CurrentFightPhase = self.FightPhase.EVACUATEROCKET;
	
	self.GenericTimer:Reset();
	self.DropShipHealthDrainTimer:Reset();
	self.RocketTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- Phase "Evacuate with dropship"
-----------------------------------------------------------------------------------------
function Nest:PhaseEvacuateDropShip()
	if not self.GenericTimer:IsPastSimMS(6500) then
		-- Show evacuaton marks
		--if MovableMan:IsActor(self.Dimitri) then
		--	self:AddObjectivePoint("Evacuate", self.Dimitri.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		--end

		--if MovableMan:IsActor(self.Mia) then
		--	self:AddObjectivePoint("Evacuate", self.Mia.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		--end

		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("Evacuate", self.AssignedBrains[player].AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	
		self:AddObjectivePoint("ESCAPE", Vector(650, 1), Activity.TEAM_1, GameActivity.ARROWUP);
		self:AddObjectivePoint("ESCAPE", Vector(1950, 1), Activity.TEAM_1, GameActivity.ARROWUP);
	end

	-- Show true evac marks
	if MovableMan:IsActor(self.DropShip) then
		self:AddObjectivePoint("Board ship and evacuate!!!", self.DropShip.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWUP);
	else
		if not self.Orbited then
			self.DropShip = nil;
			self:AdvanceToEvacuateRocket("DROPSHIP LOST, WAIT FOR ROCKET");
		end
	end
	
	--Increase beam firing rate with altitude
	if MovableMan:IsActor(self.DropShip) then
		if self.DropShip:HasObject("Dafred Brain") then
			local alt = SceneMan:FindAltitude(self.DropShip.Pos , 0 , 3);
					
			if alt > 250 then
				self.BeamWarningInterval = 3000;
				self.BeamFireInterval = 6000;
			elseif alt > 500 then
				self.BeamWarningInterval = 1500;
				self.BeamFireInterval = 6000;
			end
		end
	end
	
	if not self.EnableBeams then
		-- Show beam bombardment warning
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("ION BOMBARDMENT , WATCH OUT", player, 500, 3500, true);
		end					
		
		-- Wait some time vefore beam bombardment
		if self.GenericTimer:IsPastSimMS(6500) then
			self.EnableBeams = true;
			self.BeamTargeted = false;			
			self.BeamTimer:Reset();
		end
	else
		-- Show laser warning
		if self.BeamTimer:IsPastSimMS(self.BeamWarningInterval) then
			if not self.BeamTargeted then
				self.BeamTarget = self.RocketLZ:GetRandomPoint().X;
				if MovableMan:IsActor(self.DropShip) then
					self.BeamTarget = self.DropShip.Pos.X;
				else
					for player = 0, self.PlayerCount - 1 do
						if MovableMan:IsActor(self.AssignedBrains[player]) then
							self.BeamTarget = self.AssignedBrains[player].Pos.X;
							break;
						end
					end --for
				end -- else
				self.BeamTargeted = true;
			end
			
			self:AddObjectivePoint("DANGER", Vector(self.BeamTarget , 0), self.PlayerTeam, GameActivity.ARROWUP);
		end --if
			
		if self.BeamTimer:IsPastSimMS(self.BeamWarningInterval + self.BeamFireInterval) then
			self:FireBeam(self.BeamTarget);
			self.BeamTargeted = false;
			self.BeamTimer:Reset();
		end
		
		if  self.BeamEmitterTimer:IsPastSimMS(self.BeamEmitterTime) then
			self.BeamEmitter:EnableEmission(false);
		end
	end
end
-----------------------------------------------------------------------------------------
-- Phase "Evacuate with rocket"
-----------------------------------------------------------------------------------------
function Nest:PhaseEvacuateRocket()
	if not self.GenericTimer:IsPastSimMS(6500) then
		-- Show evacuaton marks
		--if MovableMan:IsActor(self.Dimitri) then
		--	self:AddObjectivePoint("Evacuate", self.Dimitri.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		--end

		--if MovableMan:IsActor(self.Mia) then
		--	self:AddObjectivePoint("Evacuate", self.Mia.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		--end

		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("Evacuate", self.AssignedBrains[player].AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	
		self:AddObjectivePoint("ESCAPE", Vector(650, 1), Activity.TEAM_1, GameActivity.ARROWUP);
		self:AddObjectivePoint("ESCAPE", Vector(1950, 1), Activity.TEAM_1, GameActivity.ARROWUP);
	end

	-- Show true evac marks
	if MovableMan:IsActor(self.Rocket) then
		self:AddObjectivePoint("Board rocket and evacuate!!!", self.Rocket.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWUP);
		self.RocketTimer:Reset();
	else
		--self.Rocket = nil;
		if self.RocketTimer:IsPastSimMS(self.RocketInterval) then
			self:SpawnRocket();
			self.RocketTimer:Reset();
		end
	end
	
	-- Decrease dropship health if we lost robot
	if MovableMan:IsActor(self.DropShip) then
		if self.DropShipHealthDrainTimer:IsPastSimMS(150) then
			self.DropShip.Health = self.DropShip.Health - 1;
			self.DropShipHealthDrainTimer:Reset();
		end
		
		self:ControlShip();
	else
		self.LeftEmitter:EnableEmission(false);
		self.RightEmitter:EnableEmission(false);
		self.DropShip = nil;
	end

	--Increase beam firing rate with altitude
	if MovableMan:IsActor(self.Rocket) then
		if self.Rocket:HasObject("Dafred Brain") then
			local alt = SceneMan:FindAltitude(self.Rocket.Pos , 0 , 3);
					
			if alt > 250 then
				self.BeamWarningInterval = 3000;
				self.BeamFireInterval = 6000;
			elseif alt > 500 then
				self.BeamWarningInterval = 1500;
				self.BeamFireInterval = 6000;
			end
		end
	end
	
	if not self.EnableBeams then
		if self.GenericTimer:IsPastSimMS(5500) then
			-- Show beam bombardment warning
			for player = 0, self.PlayerCount - 1 do
				self:ResetMessageTimer(player);
				FrameMan:ClearScreenText(player);
				FrameMan:SetScreenText("ION BOMBARDMENT , WATCH OUT", player, 500, 3500, true);
			end					
		end
		
		-- Wait some time vefore beam bombardment
		if self.GenericTimer:IsPastSimMS(6500) then
			self.EnableBeams = true;
			self.BeamTargeted = false;
			self.BeamTimer:Reset();
		end
	else
		-- Show ion warning
		if self.BeamTimer:IsPastSimMS(self.BeamWarningInterval) then
			if not self.BeamTargeted then
				self.BeamTarget = self.RocketLZ:GetRandomPoint().X;
				if MovableMan:IsActor(self.Rocket) then
					self.BeamTarget = self.Rocket.Pos.X;
				else
					for player = 0, self.PlayerCount - 1 do
						if MovableMan:IsActor(self.AssignedBrains[player]) then
							self.BeamTarget = self.AssignedBrains[player].Pos.X;
							break;
						end
					end --for
				end -- else
				self.BeamTargeted = true;
				print ("Beam targeted");
			end
			
			self:AddObjectivePoint("DANGER", Vector(self.BeamTarget , 0), self.PlayerTeam, GameActivity.ARROWUP);
		end --if
			
		if self.BeamTimer:IsPastSimMS(self.BeamWarningInterval + self.BeamFireInterval) then
			self:FireBeam(self.BeamTarget);
			self.BeamTargeted = false;
			self.BeamTimer:Reset();
			print ("Beam fired");
		end
		
		if  self.BeamEmitterTimer:IsPastSimMS(self.BeamEmitterTime) then
			self.BeamEmitter:EnableEmission(false);
		end
	end
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function Nest:CraftEnteredOrbit()
	--If a craft leaves with the brain robot...
	if self.OrbitedCraft:HasObject("Dafred Brain") then
		if self.CurrentFightPhase == self.FightPhase.EVACUATEDROPSHIP or self.CurrentFightPhase == self.FightPhase.EVACUATEROCKET then
			-- Team 1 wins!
			self.WinnerTeam = self.PlayerTeam;
			ActivityMan:EndActivity();
			self.Orbited = true;
			-- Tell the script to stop running for brains as the game has ended.
			for player = 0, self.PlayerCount - 1 do
				self.braindead[player] = true;
			end
		else
			for player = 0, self.PlayerCount - 1 do
				self:ResetMessageTimer(player);
				FrameMan:ClearScreenText(player);
				FrameMan:SetScreenText("Coward!!!", player, 1500, 10000, true);
			end
		end
	else
		print ("Ship empty");
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------








































